/*  This file is part of gpkgdep.

    gpkgdep is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    gpkgdep is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with gpkgdep; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <sys/param.h>
#include <stdio.h>
#include "access_layer.h"
#include <string.h>

#ifdef BSD
GSList * loadPackageNames()
{
   GSList * result = g_slist_alloc();
   FILE * f = popen("pkg_info", "r");
   char s[1024];
   int len;
   char * end, * start;
   GString * line = g_string_new(NULL);
   packageNameAndDescription * p;

   for(;;)
   {
    if (!fgets(s, sizeof(s), f))
    {
            pclose(f);
      g_string_free(line, TRUE);
      return result;
    }
    len = strlen(s);
    if (len == 0 || s[len-1] == '\n')
    {
      s[len-1] = 0;
      g_string_append(line, s);
      end = strchr(line->str, ' ');
      p = (packageNameAndDescription *)g_malloc(sizeof(packageNameAndDescription));
      if (end)
      {
        start = end+1;
        while (*start && *start == ' ')
            ++start;
        p->description = g_string_new(start);
        g_string_truncate(line, end-line->str);
      }
      else
          p->description = g_string_new(NULL);
      p->name = line;
      line = g_string_new(NULL);
      result = g_slist_prepend(result, p);
    }
    else
      g_string_append(line, s);
   }
   pclose(f);
   return result;
 }
#else
int readline(GString ** line, FILE *f)
{
   char s[1024];
   int len;
   g_string_assign(*line, "");
   for(;;)
   {
    if (!fgets(s, sizeof(s), f))
      return 0;
    len = strlen(s);
    if (len == 0 || s[len-1] == '\n')
    {
      s[len-1] = 0;
      g_string_append(*line, s);
      return -1;
    }
    else
      g_string_append(*line, s);
   }
}

GSList * loadPackageNames()
{
   GSList * result = g_slist_alloc();
   FILE * f = popen("qpkg -i -I -nc", "r");
   char * end, * start;
   GString * line = g_string_new(NULL);
   packageNameAndDescription * p;

   for(;;)
   {
    if (!readline(& line, f))
    {
      pclose(f);
      g_string_free(line, TRUE);
      return result;
    }
    end = strchr(line->str, ' ');
    p = (packageNameAndDescription *)g_malloc(sizeof(packageNameAndDescription));
    if (end)
      g_string_truncate(line, end-line->str);
    p->description = NULL;
    start = strchr(line->str, '/');
    if (start)
    {
      p->name = g_string_new(start+1); 
      g_string_truncate(line, start-line->str);
      p->description = line;
    }
    else
      p->name = line;
    line = g_string_new(NULL);
    if (!readline(& line, f))
    {
      pclose(f);
      g_string_free(line, TRUE);
      return result;
    }
    start = line->str;
    while (*start && (*start == ' ' || *start == '\t'))
      ++start;
    if (p->description)
    {
      g_string_append(p->description, ": ");
      g_string_append(p->description, start);
    }
    else
      p->description = g_string_new(start);
    result = g_slist_prepend(result, p);
   }
   pclose(f);
   return result;
 }
#endif
 
void deletePackageNameAndDescriptionItem (gpointer data, gpointer user_data)
{
    packageNameAndDescription * p = (packageNameAndDescription *)data;
    if (!p)
        return;
    if (p->name)
        g_string_free(p->name, TRUE);
    if (p->description)
        g_string_free(p->description, TRUE);
}

void deletePackageNames(GSList * pkgList)
{
    g_slist_foreach(pkgList, deletePackageNameAndDescriptionItem, NULL);
    g_slist_free(pkgList);
}

#ifdef BSD
GSList * loadChildPackageNames(gchar * packageName)
{
    GSList * result = g_slist_alloc();
    FILE * f;
    char s[1024], * end;
    GString * line = g_string_new(NULL);
    int len;

    sprintf(s, "pkg_info -rq %s", packageName);
    f = popen(s, "r");
    for(;;)
    {
        if (!fgets(s, sizeof(s), f))
        {
            pclose(f);
            g_string_free(line, TRUE);
            return result;
        }
        len = strlen(s);
        if (len == 0 || s[len-1] == '\n')
        {
            s[len-1] = 0;
            g_string_append(line, s);
            end = strchr(line->str, ' ');
            if (end)
            {
                if (strncmp(line->str, "@pkgdep", 7))
                    continue;
                g_string_erase(line, 0, end-line->str+1);
            }
            else
                continue;
            result = g_slist_prepend(result, line->str);
            g_string_free(line, FALSE);
            line = g_string_new(NULL);
        }
        else
            g_string_append(line, s);
   }
   pclose(f);
   return result;
 }
#else
GSList * loadChildPackageNames(gchar * packageName)
{
    GSList * result = g_slist_alloc();
    FILE * f;
    char s[1024], * start;
    GString * line = g_string_new(NULL);
    int len;

    sprintf(s, "qpkg -q -nc -I %s", packageName);
    f = popen(s, "r");
    for(;;)
    {
        if (!fgets(s, sizeof(s), f))
        {
            pclose(f);
            g_string_free(line, TRUE);
            return result;
        }
        len = strlen(s);
        if (len == 0 || s[len-1] == '\n')
        {
            s[len-1] = 0;
            g_string_append(line, s);
            start = line->str;
            if (*start != ' ' && *start != '\t')
            {
              g_string_truncate(line, 0);
              continue;
            }
            start = strchr(start, '/');
            g_string_erase(line, 0, start - line->str+1);
            result = g_slist_prepend(result, line->str);
            g_string_free(line, FALSE);
            line = g_string_new(NULL);
        }
        else
            g_string_append(line, s);
   }
   pclose(f);
   return result;
 }
#endif

 
void deleteChildPackageNames(GSList * pkgList)
{
    g_slist_free(pkgList);
}
