/*  This file is part of gpkgdep.

    gpkgdep is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    gpkgdep is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with gpkgdep; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "gtk/gtk.h"
#include "dep_pkg_tree.h"
#include "package_tree.h"

GtkTreeStore * dependentTreeModel = NULL;
GtkWidget * dependentTreeWidget = NULL;

void dependentTreeProcessChilds(gpointer data, gpointer user_data);

void dependent_tree_row_expanded_event (GtkTreeView *treeview, GtkTreeIter *arg1, GtkTreePath *arg2, gpointer user_data)
{
    GtkTreeModel * model = gtk_tree_view_get_model(treeview);
    package * p;
    GtkTreeIter iter;
    gboolean b = gtk_tree_model_iter_children(model, & iter, arg1);
    if (!b)
        return;
    while(1)
    {
        gtk_tree_model_get(model, & iter, 2, & p, -1);
        b = gtk_tree_model_iter_has_child(model, & iter);
        if (!b)
            g_slist_foreach(p->parents, dependentTreeProcessChilds, & iter);
        b = gtk_tree_model_iter_next(model, &iter);
        if (!b)
            break;
    }
}

void dependentTreeProcessChilds(gpointer data, gpointer user_data)
{
    package * p = (package *)data;
    GtkTreeIter * iter = (GtkTreeIter *) user_data;
    GtkTreeIter childIter;
    if (!p)
        return;
    gtk_tree_store_append(dependentTreeModel, & childIter, iter);  /* Acquire an iterator */
    gtk_tree_store_set(dependentTreeModel, & childIter, 0, p->name->str, 1, "", 2, p, -1);
    return;
}

void addDepPackageTreeItem(gpointer key, gpointer value, gpointer user_data)
{
    package * p = (package *)value;
    GtkTreeIter iter;
    if (!p)
        return;
    gtk_tree_store_append(dependentTreeModel, & iter, NULL);  /* Acquire an iterator */
    gtk_tree_store_set(dependentTreeModel, &iter, 0, p->name->str, 1, p->description->str, 2, p, -1);
    g_slist_foreach(p->parents, dependentTreeProcessChilds, & iter);
    return;
}

void fillDependentTree()
{
    g_hash_table_foreach(packageTree, addDepPackageTreeItem, NULL);
}

GtkWidget * createDependentTreeWidget()
{
    GtkWidget * tree;
    GtkTreeViewColumn *column;

    if (dependentTreeModel)
        return dependentTreeWidget;
    dependentTreeModel = gtk_tree_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
    tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL (dependentTreeModel));
    g_object_unref (G_OBJECT (dependentTreeModel));
    column = gtk_tree_view_column_new_with_attributes("Name", gtk_cell_renderer_text_new(), "text", 0, NULL);
#if GTK_CHECK_VERSION(2,0,9)
    gtk_tree_view_column_set_sort_indicator(column,TRUE);
    gtk_tree_view_column_set_sort_column_id(column,0);
#endif
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column(GTK_TREE_VIEW (tree), column);
    column = gtk_tree_view_column_new_with_attributes("Description", gtk_cell_renderer_text_new(), "text", 1, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW (tree), column);

    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(dependentTreeModel), 0, GTK_SORT_ASCENDING);

    dependentTreeWidget = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_set_border_width (GTK_CONTAINER (dependentTreeWidget), 5);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (dependentTreeWidget), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    gtk_container_add (GTK_CONTAINER (dependentTreeWidget), GTK_WIDGET (tree));

    gtk_widget_show(GTK_WIDGET(tree));

    g_signal_connect (G_OBJECT (tree), "row-expanded", G_CALLBACK (dependent_tree_row_expanded_event), NULL);
    gtk_widget_show(dependentTreeWidget);
    return dependentTreeWidget;
}



