/*  This file is part of gpkgdep.

    gpkgdep is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    gpkgdep is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with gpkgdep; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <sys/param.h>
#include <stdio.h>
#include "access_layer.h"
#include "package_tree.h"
#include <string.h>

GHashTable * packageTree = NULL;
GSList * invalidLinks = NULL;


package * newPackage()
{
    package * newPackage = (package *)g_malloc(sizeof(package));
    newPackage->name = NULL;
    newPackage->description = NULL;
    newPackage->childs = g_slist_alloc();
    newPackage->parents = g_slist_alloc();
    newPackage->hasInvalidDependence = 0;
    newPackage->markedAsKeep = FALSE;
    newPackage->dependentKeepCount = 0;
    newPackage->userData = NULL;
    return newPackage;
}

void deletePackage(package * p, int deleteContentsOnly)
{
    if (p->name)
        g_string_free(p->name, TRUE);
    if (p->description)
        g_string_free(p->description, TRUE);
    if (p->childs)
        g_slist_free(p->childs);
    if (p->parents)
        g_slist_free(p->parents);
    if (deleteContentsOnly)
        return;
    g_free(p);
}


void processPackageNames(gpointer data, gpointer user_data)
{
    packageNameAndDescription * p = (packageNameAndDescription *)data;
    package * new_Package;

    if (!p)
        return;
    new_Package = newPackage();
    new_Package->name = p->name;
    p->name = NULL;
    new_Package->description = p->description;
    p->description = NULL;
    g_hash_table_insert(packageTree, new_Package->name->str, new_Package);
}

void setInvalidDependenceFlag(gpointer data, gpointer user_data)
{
    package * p = (package *)data;
    if (!p)
        return;
    p->hasInvalidDependence = TRUE;
    g_slist_foreach(p->parents, setInvalidDependenceFlag, NULL);
}

void processPackageChilds(gpointer data, gpointer user_data)
{
    gchar * childName = (gchar *)data;
    package * p = (package *)user_data;
    package * child;

    if (!p || !childName)
        return;
    child = g_hash_table_lookup(packageTree, childName);
    if(!child)
    {
        child = newPackage();
        child->name = g_string_new(childName);
        if (!invalidLinks)
            invalidLinks = g_slist_alloc();
        g_slist_prepend(invalidLinks, child);
        setInvalidDependenceFlag(child, NULL);
        setInvalidDependenceFlag(p, NULL);
    }
#ifdef BSD    
    p->childs = g_slist_prepend(p->childs, child);
    child->parents = g_slist_prepend(child->parents, p);
#else
    child->childs = g_slist_prepend(child->childs, p);
    p->parents = g_slist_prepend(p->parents, child);
#endif
}

void getAllPackages(gpointer key, gpointer value, gpointer user_data)
{
    package * p = (package *)value;
    GSList ** result = (GSList **) user_data;
    if (!p || !result)
        return;
    *result = g_slist_prepend(*result, p);
}

void loadPackageChilds(package * p)
{
    GSList * result;

    if (!p)
        return;
    result = loadChildPackageNames(p->name->str);
    g_slist_foreach(result, processPackageChilds, p);
    deleteChildPackageNames(result);
}

gboolean loadPackageTree(int (*loadPackagesCallback)(int step, int maxStep, gpointer data), gpointer data)
{
    GSList * result;
    GSList * list;
    int pkgNum, pkgCount = 0;
    
    if (packageTree)
        deletePackageTree();
    packageTree = g_hash_table_new(g_str_hash, g_str_equal);
    result = loadPackageNames();
    g_slist_foreach(result, processPackageNames, NULL);
    deletePackageNames(result);

    result = g_slist_alloc();
    g_hash_table_foreach(packageTree, getAllPackages, & result);
    pkgNum = g_slist_length(result);
    for (list = result; list; list = g_slist_next(list))
    {
        package * p = (package *)list->data;
        if (p)
            loadPackageChilds(p);
        if (loadPackagesCallback)
            if (loadPackagesCallback(pkgCount++, pkgNum, data))
            {
                deletePackageTree();
                g_slist_free(result);
                return FALSE;
            }
    }
    g_slist_free(result);
    return TRUE;
}

gboolean deletePackageTreeItem(gpointer key, gpointer value, gpointer user_data)
{
    package * p = (package *)value;
    if (!p)
        return TRUE;
    deletePackage(p, TRUE);
    return TRUE;
}

void deleteInvalidLink(gpointer data, gpointer user_data)
{
    package * p = (package *)data;
    if (!p)
        return;
    deletePackage(p, TRUE);
}

void deletePackageTree()
{
    if (!packageTree)
        return;
    g_hash_table_foreach_remove(packageTree, deletePackageTreeItem, NULL);
    g_hash_table_destroy(packageTree);
    packageTree = NULL;
    if (!invalidLinks)
        return;
    g_slist_foreach(invalidLinks, deleteInvalidLink, NULL);
    g_slist_free(invalidLinks);
    invalidLinks = NULL;
}

int numberOfInvalidLinks()
{
    if (!invalidLinks)
        return 0;
    return g_slist_length(invalidLinks);
}


void doPackageMarkedAsKeep(gpointer data, gpointer user_data)
{
    package * p = (package *)data;
    GSList ** result = (GSList **) user_data, *list;

    if (!p || !result)
        return;
    for (list = *result; list; list = g_slist_next(list))
    {
        package * pkg = (package *)list->data;
        if (pkg == p)
            return;
    }
    ++p->dependentKeepCount;
    *result = g_slist_prepend(*result, p);
    g_slist_foreach(p->childs, doPackageMarkedAsKeep, result);
}

GSList * packageMarkedAsKeep(package * p)
{
    GSList * result = g_slist_alloc();
    p->markedAsKeep = TRUE;
    ++p->dependentKeepCount;
    result = g_slist_prepend(result, p);
    g_slist_foreach(p->childs, doPackageMarkedAsKeep, & result);
    return result;
}

void doPackageUnmarkedAsKeep(gpointer data, gpointer user_data)
{
    package * p = (package *)data;
    GSList ** result = (GSList **) user_data, * list;

    if (!p || !result)
        return;
    for (list = *result; list; list = g_slist_next(list))
    {
        package * pkg = (package *)list->data;
        if (pkg == p)
            return;
    }
    --p->dependentKeepCount;
    if (p->dependentKeepCount < 0)
        printf("internal error in doPackageUnmarkedAsKeep: %d, %s", p->dependentKeepCount, p->name->str);
    *result = g_slist_prepend(*result, p);
    g_slist_foreach(p->childs, doPackageUnmarkedAsKeep, result);
}

GSList * packageUnarkedAsKeep(package * p)
{
    GSList * result = g_slist_alloc();
    p->markedAsKeep = FALSE;
    --p->dependentKeepCount;
    result = g_slist_prepend(result, p);
    g_slist_foreach(p->childs, doPackageUnmarkedAsKeep, & result);
    return result;
}
