/*  This file is part of gpkgdep.

    gpkgdep is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    gpkgdep is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with gpkgdep; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "gtk/gtk.h"
#include "pkg_simul.h"
#include "package_tree.h"

GtkTreeStore * simulatorTreeModel = NULL;
GtkWidget * simulatorTreeWidget = NULL;
char *displayModeNames[] = {
   "Show all packages",
   "Show wanted and unwanted packages",
   "Show unwanted packages",
   "Show wanted packages",
   "Show wanted and dependent packages",
   "Show dependent packages"};
int displayMode = 2;

void updateListEntry(gpointer data, gpointer user_data)
{
    GtkTreeIter iter;
		GtkTreeRowReference * pGtkTrRwRef;
		GtkTreePath * pGtkTrPth;
    package * p = (package *)data;
    int remove = FALSE;

    if (!p)
        return;
    switch(displayMode)
    {
        case 0:
            break;
        case 1:
            if (p->dependentKeepCount && !p->markedAsKeep)
                remove = TRUE;
            break;
        case 2:
            if (p->dependentKeepCount)
                remove = TRUE;
            break;
        case 3:
            if (!p->markedAsKeep)
                remove = TRUE;
            break;
        case 4:
            if (!p->dependentKeepCount)
                remove = TRUE;
            break;
        case 5:
            if (!p->dependentKeepCount || p->markedAsKeep)
                remove = TRUE;
            break;
    }
    pGtkTrRwRef = p->userData;
    if (!pGtkTrRwRef)
    {
        if (remove)
            return;
        gtk_tree_store_append(simulatorTreeModel, & iter, NULL);
				pGtkTrPth = gtk_tree_model_get_path(GTK_TREE_MODEL(simulatorTreeModel), & iter);
				pGtkTrRwRef = gtk_tree_row_reference_new(GTK_TREE_MODEL(simulatorTreeModel),pGtkTrPth);
				gtk_tree_path_free(pGtkTrPth);
    }
    else
    {
        if (remove)
        {
					  pGtkTrPth = gtk_tree_row_reference_get_path(pGtkTrRwRef);
					  gtk_tree_model_get_iter(GTK_TREE_MODEL(simulatorTreeModel),& iter,pGtkTrPth);
            gtk_tree_store_remove(simulatorTreeModel, & iter);
						gtk_tree_path_free(pGtkTrPth);
						gtk_tree_row_reference_free(pGtkTrRwRef);
            p->userData = NULL;
            return;
        }
    }
    gtk_tree_store_set(simulatorTreeModel, & iter, 0, p->dependentKeepCount, 1, p->name->str, 2, p->description->str, 3, p, 4, p->hasInvalidDependence?"#ff0000":"#000000", 5, !p->dependentKeepCount || p->markedAsKeep, -1);
    p->userData = pGtkTrRwRef;
}

void comboChanged (GtkWidget *widget, gpointer data)
{
    int i = gtk_option_menu_get_history (GTK_OPTION_MENU (widget));
    if (i == displayMode)
        return;
    displayMode = i;
    fillPackageSimulatorTree();
}

void tree_row_toggled_event (GtkCellRendererToggle * celltoggle, gchar * path_string, GtkTreeView * treeView)
{
    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeView));
    GtkTreePath *path;
    package * p;
    GtkTreeIter iter;
    gboolean active = FALSE;
    GSList * changedPkgs;

    path = gtk_tree_path_new_from_string (path_string);
    if (!gtk_tree_model_get_iter (model, & iter, path))
    {
        g_warning ("%s: bad path?", G_STRLOC);
        return;
    }
    gtk_tree_model_get (GTK_TREE_MODEL (model), & iter, 0, & active, -1);

    gtk_tree_store_set (GTK_TREE_STORE (model), & iter, 0, ! active, -1);

    gtk_tree_model_get(model, & iter, 3, & p, -1);
    if (active)
        changedPkgs = packageUnarkedAsKeep(p);
    else
        changedPkgs = packageMarkedAsKeep(p);
    g_slist_foreach(changedPkgs, updateListEntry, NULL);
    g_slist_free(changedPkgs);
}


void addSimulPackageTreeItem(gpointer key, gpointer value, gpointer user_data)
{
    package * p = (package *)value;
    if (!p)
        return;
    updateListEntry(p, NULL);
}

void removeUserData(gpointer key, gpointer value, gpointer user_data)
{
    package * p = (package *)value;
    if (!p)
        return;
    p->userData = NULL;
}

void fillPackageSimulatorTree()
{
    g_hash_table_foreach(packageTree, removeUserData, NULL);
    gtk_tree_store_clear(simulatorTreeModel);
    g_hash_table_foreach(packageTree, addSimulPackageTreeItem, NULL);
}

GtkWidget * createPackageSimulatorWidget()
{
    GtkWidget * tree;
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;
    GtkCellRenderer * cellRenderer;
    GtkWidget * treeWidget;
    GtkWidget * menu, * box;
    GtkWidget * displayModeCombo;
    int i = 0;

    if (simulatorTreeWidget)
        return simulatorTreeWidget;
    simulatorTreeModel = gtk_tree_store_new (6, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_STRING, G_TYPE_BOOLEAN);
    tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL (simulatorTreeModel));
    cellRenderer = gtk_cell_renderer_toggle_new();
    gtk_cell_renderer_toggle_set_active(GTK_CELL_RENDERER_TOGGLE(cellRenderer), TRUE);
    column = gtk_tree_view_column_new_with_attributes("OK", cellRenderer, "active", 0, "activatable", 5, NULL);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column(GTK_TREE_VIEW (tree), column);

    column = gtk_tree_view_column_new_with_attributes("Name", gtk_cell_renderer_text_new(), "text", 1, "foreground", 4, NULL);
    gtk_tree_view_column_set_sort_indicator(column,TRUE);
    gtk_tree_view_column_set_sort_column_id(column,1);
    gtk_tree_view_column_set_resizable(column, TRUE);
    gtk_tree_view_append_column(GTK_TREE_VIEW (tree), column);

    column = gtk_tree_view_column_new_with_attributes("Description", gtk_cell_renderer_text_new(), "text", 2, "foreground", 4, NULL);
    gtk_tree_view_append_column(GTK_TREE_VIEW (tree), column);

    treeWidget = gtk_scrolled_window_new (NULL, NULL);
    gtk_container_set_border_width (GTK_CONTAINER (treeWidget), 5);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (treeWidget), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    gtk_container_add (GTK_CONTAINER (treeWidget), GTK_WIDGET (tree));

    gtk_widget_show(GTK_WIDGET(tree));

    g_signal_connect (G_OBJECT (cellRenderer), "toggled", G_CALLBACK (tree_row_toggled_event), tree);
    simulatorTreeWidget = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (simulatorTreeWidget), treeWidget, TRUE, TRUE, 0);

    menu = gtk_menu_new ();
    displayModeCombo = gtk_option_menu_new ();
    for (i = 0; i < sizeof(displayModeNames)/sizeof(displayModeNames[0]); i++)
    {
        gtk_menu_shell_append(GTK_MENU_SHELL(menu), gtk_menu_item_new_with_label(displayModeNames[i]));
    }
    gtk_option_menu_set_menu (GTK_OPTION_MENU (displayModeCombo), menu);
    gtk_option_menu_set_history (GTK_OPTION_MENU (displayModeCombo), displayMode);
    gtk_signal_connect(GTK_OBJECT(displayModeCombo), "changed", GTK_SIGNAL_FUNC (comboChanged), NULL);

    box = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (box), 10);
    gtk_box_pack_start (GTK_BOX (box), gtk_label_new("Display filter: "), FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box), displayModeCombo, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (simulatorTreeWidget), box, FALSE, FALSE, 0);
    gtk_widget_show_all(simulatorTreeWidget);
    return simulatorTreeWidget;
}



